unit diag;

Interface
uses
  SysUtils, Controls, Dialogs, StdCtrls, oemath;

procedure diago(var mat,mat_vec_prop:matrice3; var val_prop:vecteur3);

Implementation
type
    ShortVector  =          ARRAY[0..2] of double;
    Recmat       =          ARRAY[0..2] of ShortVector;

PROCEDURE CyclicJacobi(VAR a: recmat;
                           n: integer;
                       VAR eigenvalues: shortVector;
                       VAR v: recmat;
                       VAR count: integer;
                       VAR success: boolean);

(*const nearzero = 1.0e-8;  {change to 1.0e-15 for 8087 system}
      tolerance = 1.0e-8;*)
const nearzero = 1.0e-15;  {change to 1.0e-15 for 8087 system}
      tolerance = 1.0e-15;

VAR maxCount, i,p,q : integer;
    PiOver4, PiOver2, sumsq: double;
    avp, avq: shortvector;

{--------------------------------------------------------------------------}
PROCEDURE JacobiRotation(VAR m: recmat;
                 p,q: integer;
            VAR mp, mq: shortvector);

VAR  g,h,mpp, mqq, mpq, mpj, mqj,
     tau,tempr, angle, c,s,cs,cSquared,sSquared: double;
     j: integer;

begin
  mpp := mp[p]; mpq := mp[q]; mqq := mq[q];
  if abs(mpp-mqq) > nearzero then
  begin
    angle := arctan(2.0 * mpq/(mpp-mqq))/2.0;
    if angle > PiOver4 then
      angle := angle - PiOver2
  end else
    if mpq >0 then angle := PiOver4 else angle := -PiOver4;

  c := cos(angle) ; s := sin(angle);
  cSquared := c*c; sSquared := s*s ; cs := c * s;
  tempr := 2.0 * mpq * cs;
  mp[p] := mpp * cSquared + tempr + mqq * sSquared;
  mq[q] := mpp * sSquared - tempr + mqq * cSquared;
  mp[q] := 0.0 ; mq[p] := 0.0;
  tau := s / (1.0 + c);
  for j := 0 to n-1 do
    if not(j in [p,q] ) then
    begin
      mpj := mp[j] * c + mq[j] * s;
      mqj := mq[j] * c - mp[j] * s;
      mp[j] := mpj; m[j,p] := mpj;
      mq[j] := mqj; m[j,q] := mqj;
    end;
    for j := 0 to n-1 do
    begin
      g := v[j,p]; h := v[j,q];
      v[j,p] := c * g  + s * h;
      v[j,q] := -s * g +  c * h;
    end;

end;
{--------------------------------------------------------------------------}
begin
  maxCount :=  count;
  count := 0;
  for p := 0 to n-1 do begin
    for q := 0 to n-1 do v[p,q] := 0.0;
    v[p,p] := 1.0;
  end;

  PiOver4 := pi/4.0; PiOver2 := pi/2.0;
  repeat
    count := count + 1;
    sumsq := 0.0;
    for p := 0 to n-1 do sumsq := sumsq + a[p,p] * a[p,p];
    success := true;
    for p := 0 to n -2  do
      for q := p + 1 to n-1 do
        if abs(a[p,q]/sumsq) > tolerance then begin
          success := false;
          JacobiRotation(a,p,q,a[p],a[q]);
        end;
  until success or (count > maxCount);
  if success then begin
    for p := 0 to n-1 do eigenvalues[p] := a[p,p];
  end;
end;

{--------------------------------------------------------------------------}

procedure diago(var mat,mat_vec_prop:matrice3; var val_prop:vecteur3);{
------------------------------------------------------------------
Diagonalisation de matrice 3X3}


VAR
       i,j,k,n,count  : integer;
       a,ev           : recmat;
       eigenvalues    : shortvector;
       success        : boolean;
       x              : double;

begin
   n := 3;
   count := 10;
   for i:=1 to 3 do begin
     for j:=1 to 3 do a[i-1,j-1]:=mat[i,j];
   end;
   CyclicJacobi(a,n, eigenvalues,ev,count,success);
   if success then begin
     for i:=1 to 3 do val_prop[i]:=eigenvalues[i-1];
     for i:=1 to 3 do begin
       for j:=1 to 3 do mat_vec_prop[i,j]:=ev[i-1,j-1];
     end;
   end;
   {Rangement en valeurs croissantes}
   for i:=1 to 2 do begin
     for j:=i+1 to 3 do begin
       if (val_prop[i] < val_prop[j]) then begin
         x:=val_prop[j];
         val_prop[j]:=val_prop[i];
         val_prop[i]:=x;
         for k:=1 to 3 do begin
           x:=mat_vec_prop[k,i];
           mat_vec_prop[k,i]:=mat_vec_prop[k,j];
           mat_vec_prop[k,j]:=x;
         end;
       end;
     end;
   end;
end;

end.
